@@ -2,7 +2,7 @@ use crate::{config::ResourceType, model::CachedMember, InMemoryCache, UpdateCach
 use std::borrow::Cow;
 use twilight_model::{
     application::interaction::application_command::InteractionMember,
-    gateway::payload::{MemberAdd, MemberChunk, MemberRemove, MemberUpdate},
+    gateway::payload::incoming::{MemberAdd, MemberChunk, MemberRemove, MemberUpdate},
     guild::{Member, PartialMember},
     id::{GuildId, UserId},
 };
@@ -22,7 +22,7 @@ impl InMemoryCache {
         let member_id = member.user.id;
         let id = (guild_id, member_id);
 
-        if let Some(m) = self.0.members.get(&id) {
+        if let Some(m) = self.members.get(&id) {
             if *m == member {
                 return;
             }
@@ -42,9 +42,8 @@ impl InMemoryCache {
             roles: member.roles,
             user_id,
         };
-        self.0.members.insert(id, cached);
-        self.0
-            .guild_members
+        self.members.insert(id, cached);
+        self.guild_members
             .entry(guild_id)
             .or_default()
             .insert(member_id);
@@ -58,14 +57,13 @@ impl InMemoryCache {
     ) {
         let id = (guild_id, user_id);
 
-        if let Some(m) = self.0.members.get(&id) {
+        if let Some(m) = self.members.get(&id) {
             if *m == member {
                 return;
             }
         }
 
-        self.0
-            .guild_members
+        self.guild_members
             .entry(guild_id)
             .or_default()
             .insert(user_id);
@@ -81,7 +79,7 @@ impl InMemoryCache {
             roles: member.roles.to_owned(),
             user_id,
         };
-        self.0.members.insert(id, cached);
+        self.members.insert(id, cached);
     }
 
     pub(crate) fn cache_borrowed_interaction_member(
@@ -91,14 +89,13 @@ impl InMemoryCache {
     ) {
         let id = (guild_id, member.id);
 
-        let (deaf, mute) = match self.0.members.get(&id) {
+        let (deaf, mute) = match self.members.get(&id) {
             Some(m) if *m == member => return,
-            Some(m) => (m.deaf, m.mute),
+            Some(m) => (m.deaf(), m.mute()),
             None => (None, None),
         };
 
-        self.0
-            .guild_members
+        self.guild_members
             .entry(guild_id)
             .or_default()
             .insert(member.id);
@@ -115,7 +112,7 @@ impl InMemoryCache {
             user_id: member.id,
         };
 
-        self.0.members.insert(id, cached);
+        self.members.insert(id, cached);
     }
 }
 
@@ -128,7 +125,6 @@ impl UpdateCache for MemberAdd {
         cache.cache_member(self.guild_id, self.0.clone());
 
         cache
-            .0
             .guild_members
             .entry(self.guild_id)
             .or_default()
@@ -147,7 +143,7 @@ impl UpdateCache for MemberChunk {
         }
 
         cache.cache_members(self.guild_id, self.members.clone());
-        let mut guild = cache.0.guild_members.entry(self.guild_id).or_default();
+        let mut guild = cache.guild_members.entry(self.guild_id).or_default();
         guild.extend(self.members.iter().map(|member| member.user.id));
     }
 }
@@ -158,9 +154,9 @@ impl UpdateCache for MemberRemove {
             return;
         }
 
-        cache.0.members.remove(&(self.guild_id, self.user.id));
+        cache.members.remove(&(self.guild_id, self.user.id));
 
-        if let Some(mut members) = cache.0.guild_members.get_mut(&self.guild_id) {
+        if let Some(mut members) = cache.guild_members.get_mut(&self.guild_id) {
             members.remove(&self.user.id);
         }
 
@@ -168,14 +164,14 @@ impl UpdateCache for MemberRemove {
         // and then removing the user later if they are in no guilds.
         let mut remove_user = false;
 
-        if let Some(mut user_guilds) = cache.0.user_guilds.get_mut(&self.user.id) {
+        if let Some(mut user_guilds) = cache.user_guilds.get_mut(&self.user.id) {
             user_guilds.remove(&self.guild_id);
 
             remove_user = user_guilds.is_empty();
         }
 
         if remove_user {
-            cache.0.users.remove(&self.user.id);
+            cache.users.remove(&self.user.id);
         }
     }
 }
@@ -186,16 +182,16 @@ impl UpdateCache for MemberUpdate {
             return;
         }
 
-        let mut member = match cache.0.members.get_mut(&(self.guild_id, self.user.id)) {
+        let mut member = match cache.members.get_mut(&(self.guild_id, self.user.id)) {
             Some(member) => member,
             None => return,
         };
 
-        member.deaf = self.deaf.or(member.deaf);
-        member.mute = self.mute.or(member.mute);
+        member.deaf = self.deaf.or_else(|| member.deaf());
+        member.mute = self.mute.or_else(|| member.mute());
         member.nick = self.nick.clone();
         member.roles = self.roles.clone();
-        member.joined_at.replace(self.joined_at.clone());
+        member.joined_at.replace(self.joined_at);
         member.pending = self.pending;
     }
 }
@@ -211,26 +207,30 @@ mod tests {
 
         // Single inserts
         {
-            let guild_1_user_ids = (1..=10).map(UserId).collect::<Vec<_>>();
+            let guild_1_user_ids = (1..=10)
+                .map(|n| UserId::new(n).expect("non zero"))
+                .collect::<Vec<_>>();
             let guild_1_members = guild_1_user_ids
                 .iter()
                 .copied()
-                .map(|id| test::member(id, GuildId(1)))
+                .map(|id| test::member(id, GuildId::new(1).expect("non zero")))
                 .collect::<Vec<_>>();
 
             for member in guild_1_members {
-                cache.cache_member(GuildId(1), member);
+                cache.cache_member(GuildId::new(1).expect("non zero"), member);
             }
 
             // Check for the cached guild members ids
-            let cached_roles = cache.guild_members(GuildId(1)).unwrap();
+            let cached_roles = cache
+                .guild_members(GuildId::new(1).expect("non zero"))
+                .unwrap();
             assert_eq!(cached_roles.len(), guild_1_user_ids.len());
             assert!(guild_1_user_ids.iter().all(|id| cached_roles.contains(id)));
 
             // Check for the cached members
-            assert!(guild_1_user_ids
-                .iter()
-                .all(|id| cache.member(GuildId(1), *id).is_some()));
+            assert!(guild_1_user_ids.iter().all(|id| cache
+                .member(GuildId::new(1).expect("non zero"), *id)
+                .is_some()));
 
             // Check for the cached users
             assert!(guild_1_user_ids.iter().all(|id| cache.user(*id).is_some()));
@@ -238,24 +238,27 @@ mod tests {
 
         // Bulk inserts
         {
-            let guild_2_user_ids = (1..=10).map(UserId).collect::<Vec<_>>();
+            let guild_2_user_ids = (1..=10)
+                .map(|n| UserId::new(n).expect("non zero"))
+                .collect::<Vec<_>>();
             let guild_2_members = guild_2_user_ids
                 .iter()
                 .copied()
-                .map(|id| test::member(id, GuildId(2)))
+                .map(|id| test::member(id, GuildId::new(2).expect("non zero")))
                 .collect::<Vec<_>>();
-            cache.cache_members(GuildId(2), guild_2_members);
+            cache.cache_members(GuildId::new(2).expect("non zero"), guild_2_members);
 
             // Check for the cached guild members ids
-            let cached_roles = cache.guild_members(GuildId(1)).unwrap();
+            let cached_roles = cache
+                .guild_members(GuildId::new(1).expect("non zero"))
+                .unwrap();
             assert_eq!(cached_roles.len(), guild_2_user_ids.len());
             assert!(guild_2_user_ids.iter().all(|id| cached_roles.contains(id)));
 
             // Check for the cached members
-            assert!(guild_2_user_ids
-                .iter()
-                .copied()
-                .all(|id| cache.member(GuildId(1), id).is_some()));
+            assert!(guild_2_user_ids.iter().copied().all(|id| cache
+                .member(GuildId::new(1).expect("non zero"), id)
+                .is_some()));
 
             // Check for the cached users
             assert!(guild_2_user_ids.iter().all(|id| cache.user(*id).is_some()));
@@ -264,45 +267,51 @@ mod tests {
 
     #[test]
     fn test_cache_user_guild_state() {
-        let user_id = UserId(2);
+        let user_id = UserId::new(2).expect("non zero");
         let cache = InMemoryCache::new();
-        cache.cache_user(Cow::Owned(test::user(user_id)), Some(GuildId(1)));
+        cache.cache_user(
+            Cow::Owned(test::user(user_id)),
+            Some(GuildId::new(1).expect("non zero")),
+        );
 
         // Test the guild's ID is the only one in the user's set of guilds.
         {
-            let user_guilds = cache.0.user_guilds.get(&user_id).unwrap();
-            assert!(user_guilds.contains(&GuildId(1)));
+            let user_guilds = cache.user_guilds.get(&user_id).unwrap();
+            assert!(user_guilds.contains(&GuildId::new(1).expect("non zero")));
             assert_eq!(1, user_guilds.len());
         }
 
         // Test that a second guild will cause 2 in the set.
-        cache.cache_user(Cow::Owned(test::user(user_id)), Some(GuildId(3)));
+        cache.cache_user(
+            Cow::Owned(test::user(user_id)),
+            Some(GuildId::new(3).expect("non zero")),
+        );
 
         {
-            let user_guilds = cache.0.user_guilds.get(&user_id).unwrap();
-            assert!(user_guilds.contains(&GuildId(3)));
+            let user_guilds = cache.user_guilds.get(&user_id).unwrap();
+            assert!(user_guilds.contains(&GuildId::new(3).expect("non zero")));
             assert_eq!(2, user_guilds.len());
         }
 
         // Test that removing a user from a guild will cause the ID to be
         // removed from the set, leaving the other ID.
         cache.update(&MemberRemove {
-            guild_id: GuildId(3),
+            guild_id: GuildId::new(3).expect("non zero"),
             user: test::user(user_id),
         });
 
         {
-            let user_guilds = cache.0.user_guilds.get(&user_id).unwrap();
-            assert!(!user_guilds.contains(&GuildId(3)));
+            let user_guilds = cache.user_guilds.get(&user_id).unwrap();
+            assert!(!user_guilds.contains(&GuildId::new(3).expect("non zero")));
             assert_eq!(1, user_guilds.len());
         }
 
         // Test that removing the user from its last guild removes the user's
         // entry.
         cache.update(&MemberRemove {
-            guild_id: GuildId(1),
+            guild_id: GuildId::new(1).expect("non zero"),
             user: test::user(user_id),
         });
-        assert!(!cache.0.users.contains_key(&user_id));
+        assert!(!cache.users.contains_key(&user_id));
     }
 }
